@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2020-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Functions related to the Brave adblocker."""
 
@@ -27,7 +12,7 @@ import contextlib
 import subprocess
 from typing import Optional, IO, Iterator
 
-from PyQt5.QtCore import QUrl
+from qutebrowser.qt.core import QUrl
 
 from qutebrowser.api import (
     hook,
@@ -109,6 +94,7 @@ _RESOURCE_TYPE_STRINGS = {
     ResourceType.plugin_resource: "other",
     ResourceType.preload_main_frame: "other",
     ResourceType.preload_sub_frame: "other",
+    ResourceType.websocket: "websocket",
     ResourceType.unknown: "other",
     None: "",
 }
@@ -175,11 +161,12 @@ class BraveAdBlocker:
                     hasattr(adblock, "__file__")):
                 proc = subprocess.run(
                     ['pacman', '-Qo', adblock.__file__],
-                    stdout=subprocess.PIPE,
-                    universal_newlines=True,
+                    capture_output=True,
+                    text=True,
                     check=False,
                 )
                 logger.debug(proc.stdout)
+                logger.debug(proc.stderr)
             raise
 
     def _is_blocked(
