@@ -1,32 +1,17 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Global Qt event filter which dispatches key events."""
 
-from typing import cast
+from typing import cast, Optional
 
-from PyQt5.QtCore import pyqtSlot, QObject, QEvent
-from PyQt5.QtGui import QKeyEvent, QWindow
+from qutebrowser.qt.core import pyqtSlot, QObject, QEvent, qVersion
+from qutebrowser.qt.gui import QKeyEvent, QWindow
 
 from qutebrowser.keyinput import modeman
 from qutebrowser.misc import quitter, objects
-from qutebrowser.utils import objreg
+from qutebrowser.utils import objreg, debug, log
 
 
 class EventFilter(QObject):
@@ -35,7 +20,7 @@ class EventFilter(QObject):
 
     Attributes:
         _activated: Whether the EventFilter is currently active.
-        _handlers; A {QEvent.Type: callable} dict with the handlers for an
+        _handlers: A {QEvent.Type: callable} dict with the handlers for an
                    event.
     """
 
@@ -43,10 +28,11 @@ class EventFilter(QObject):
         super().__init__(parent)
         self._activated = True
         self._handlers = {
-            QEvent.KeyPress: self._handle_key_event,
-            QEvent.KeyRelease: self._handle_key_event,
-            QEvent.ShortcutOverride: self._handle_key_event,
+            QEvent.Type.KeyPress: self._handle_key_event,
+            QEvent.Type.KeyRelease: self._handle_key_event,
+            QEvent.Type.ShortcutOverride: self._handle_key_event,
         }
+        self._log_qt_events = "log-qt-events" in objects.debug_flags
 
     def install(self) -> None:
         objects.qapp.installEventFilter(self)
@@ -76,7 +62,7 @@ class EventFilter(QObject):
             # No window available yet, or not a MainWindow
             return False
 
-    def eventFilter(self, obj: QObject, event: QEvent) -> bool:
+    def eventFilter(self, obj: Optional[QObject], event: Optional[QEvent]) -> bool:
         """Handle an event.
 
         Args:
@@ -86,20 +72,43 @@ class EventFilter(QObject):
         Return:
             True if the event should be filtered, False if it's passed through.
         """
+        assert event is not None
+        ev_type = event.type()
+
+        if self._log_qt_events:
+            try:
+                source = repr(obj)
+            except AttributeError:  # might not be fully initialized yet
+                source = type(obj).__name__
+
+            ev_type_str = debug.qenum_key(QEvent, ev_type)
+            log.misc.debug(f"{source} got event: {ev_type_str}")
+
+        if (
+            ev_type == QEvent.Type.DragEnter and
+            objects.qapp.platformName() in ["wayland", "wayland-egl"] and
+            qVersion() == "6.5.2"
+        ):
+            # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-115757
+            # Fixed in Qt 6.5.3
+            # Can't do this via self._handlers since handling it for QWindow
+            # seems to be too late.
+            log.mouse.warning("Ignoring drag event to prevent Qt crash")
+            event.ignore()
+            return True
+
         if not isinstance(obj, QWindow):
             # We already handled this same event at some point earlier, so
             # we're not interested in it anymore.
             return False
 
-        typ = event.type()
-
-        if typ not in self._handlers:
+        if ev_type not in self._handlers:
             return False
 
         if not self._activated:
             return False
 
-        handler = self._handlers[typ]
+        handler = self._handlers[ev_type]
         try:
             return handler(cast(QKeyEvent, event))
         except:
