@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """KeyChainParser for "hint" and "normal" modes.
 
@@ -23,24 +8,32 @@ Module attributes:
     STARTCHARS: Possible chars for starting a commandline input.
 """
 
-import typing
 import traceback
 import enum
+from typing import TYPE_CHECKING, Sequence
 
-from PyQt5.QtCore import pyqtSlot, Qt, QObject
-from PyQt5.QtGui import QKeySequence, QKeyEvent
+from qutebrowser.qt.core import pyqtSlot, Qt, QObject
+from qutebrowser.qt.gui import QKeySequence, QKeyEvent
 
 from qutebrowser.browser import hints
 from qutebrowser.commands import cmdexc
 from qutebrowser.config import config
 from qutebrowser.keyinput import basekeyparser, keyutils, macros
 from qutebrowser.utils import usertypes, log, message, objreg, utils
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from qutebrowser.commands import runners
 
 
 STARTCHARS = ":/?"
-LastPress = enum.Enum('LastPress', ['none', 'filtertext', 'keystring'])
+
+
+class LastPress(enum.Enum):
+
+    """Whether the last keypress filtered a text or was part of a keystring."""
+
+    none = enum.auto()
+    filtertext = enum.auto()
+    keystring = enum.auto()
 
 
 class CommandKeyParser(basekeyparser.BaseKeyParser):
@@ -78,6 +71,8 @@ class NormalKeyParser(CommandKeyParser):
         _partial_timer: Timer to clear partial keypresses.
     """
 
+    _sequence: keyutils.KeySequence
+
     def __init__(self, *, win_id: int,
                  commandrunner: 'runners.CommandRunner',
                  parent: QObject = None) -> None:
@@ -89,6 +84,7 @@ class NormalKeyParser(CommandKeyParser):
         self._inhibited = False
         self._inhibited_timer = usertypes.Timer(self, 'normal-inhibited')
         self._inhibited_timer.setSingleShot(True)
+        self._inhibited_timer.timeout.connect(self._clear_inhibited)
 
     def __repr__(self) -> str:
         return utils.get_repr(self)
@@ -100,11 +96,11 @@ class NormalKeyParser(CommandKeyParser):
         if self._inhibited:
             self._debug_log("Ignoring key '{}', because the normal mode is "
                             "currently inhibited.".format(txt))
-            return QKeySequence.NoMatch
+            return QKeySequence.SequenceMatch.NoMatch
 
         match = super().handle(e, dry_run=dry_run)
 
-        if match == QKeySequence.PartialMatch and not dry_run:
+        if match == QKeySequence.SequenceMatch.PartialMatch and not dry_run:
             timeout = config.val.input.partial_timeout
             if timeout != 0:
                 self._partial_timer.setInterval(timeout)
@@ -118,7 +114,6 @@ class NormalKeyParser(CommandKeyParser):
                 timeout))
             self._inhibited = True
             self._inhibited_timer.setInterval(timeout)
-            self._inhibited_timer.timeout.connect(self._clear_inhibited)
             self._inhibited_timer.start()
 
     @pyqtSlot()
@@ -146,6 +141,8 @@ class HintKeyParser(basekeyparser.BaseKeyParser):
         _last_press: The nature of the last keypress, a LastPress member.
     """
 
+    _sequence: keyutils.KeySequence
+
     def __init__(self, *, win_id: int,
                  commandrunner: 'runners.CommandRunner',
                  hintmanager: hints.HintManager,
@@ -166,7 +163,7 @@ class HintKeyParser(basekeyparser.BaseKeyParser):
         """Handle keys for string filtering."""
         log.keyboard.debug("Got filter key 0x{:x} text {}".format(
             e.key(), e.text()))
-        if e.key() == Qt.Key_Backspace:
+        if e.key() == Qt.Key.Key_Backspace:
             log.keyboard.debug("Got backspace, mode {}, filtertext '{}', "
                                "sequence '{}'".format(self._last_press,
                                                       self._filtertext,
@@ -174,7 +171,7 @@ class HintKeyParser(basekeyparser.BaseKeyParser):
             if self._last_press != LastPress.keystring and self._filtertext:
                 self._filtertext = self._filtertext[:-1]
                 self._hintmanager.filter_hints(self._filtertext)
-                return QKeySequence.ExactMatch
+                return QKeySequence.SequenceMatch.ExactMatch
             elif self._last_press == LastPress.keystring and self._sequence:
                 self._sequence = self._sequence[:-1]
                 self.keystring_updated.emit(str(self._sequence))
@@ -183,18 +180,18 @@ class HintKeyParser(basekeyparser.BaseKeyParser):
                     # in numeric mode after the number has been deleted).
                     self._hintmanager.filter_hints(self._filtertext)
                     self._last_press = LastPress.filtertext
-                return QKeySequence.ExactMatch
+                return QKeySequence.SequenceMatch.ExactMatch
             else:
-                return QKeySequence.NoMatch
+                return QKeySequence.SequenceMatch.NoMatch
         elif self._hintmanager.current_mode() != 'number':
-            return QKeySequence.NoMatch
+            return QKeySequence.SequenceMatch.NoMatch
         elif not e.text():
-            return QKeySequence.NoMatch
+            return QKeySequence.SequenceMatch.NoMatch
         else:
             self._filtertext += e.text()
             self._hintmanager.filter_hints(self._filtertext)
             self._last_press = LastPress.filtertext
-            return QKeySequence.ExactMatch
+            return QKeySequence.SequenceMatch.ExactMatch
 
     def handle(self, e: QKeyEvent, *,
                dry_run: bool = False) -> QKeySequence.SequenceMatch:
@@ -205,18 +202,18 @@ class HintKeyParser(basekeyparser.BaseKeyParser):
         assert not dry_run
 
         if (self._command_parser.handle(e, dry_run=True) !=
-                QKeySequence.NoMatch):
+                QKeySequence.SequenceMatch.NoMatch):
             log.keyboard.debug("Handling key via command parser")
             self.clear_keystring()
             return self._command_parser.handle(e)
 
         match = super().handle(e)
 
-        if match == QKeySequence.PartialMatch:
+        if match == QKeySequence.SequenceMatch.PartialMatch:
             self._last_press = LastPress.keystring
-        elif match == QKeySequence.ExactMatch:
+        elif match == QKeySequence.SequenceMatch.ExactMatch:
             self._last_press = LastPress.none
-        elif match == QKeySequence.NoMatch:
+        elif match == QKeySequence.SequenceMatch.NoMatch:
             # We couldn't find a keychain so we check if it's a special key.
             return self._handle_filter_key(e)
         else:
@@ -224,7 +221,7 @@ class HintKeyParser(basekeyparser.BaseKeyParser):
 
         return match
 
-    def update_bindings(self, strings: typing.Sequence[str],
+    def update_bindings(self, strings: Sequence[str],
                         preserve_filter: bool = False) -> None:
         """Update bindings when the hint strings changed.
 
@@ -257,8 +254,10 @@ class RegisterKeyParser(CommandKeyParser):
                  mode: usertypes.KeyMode,
                  commandrunner: 'runners.CommandRunner',
                  parent: QObject = None) -> None:
-        super().__init__(mode=usertypes.KeyMode.register, win_id=win_id,
-                         commandrunner=commandrunner, parent=parent,
+        super().__init__(mode=usertypes.KeyMode.register,  # type: ignore[arg-type]
+                         win_id=win_id,
+                         commandrunner=commandrunner,
+                         parent=parent,
                          supports_count=False)
         self._register_mode = mode
 
@@ -266,12 +265,18 @@ class RegisterKeyParser(CommandKeyParser):
                dry_run: bool = False) -> QKeySequence.SequenceMatch:
         """Override to always match the next key and use the register."""
         match = super().handle(e, dry_run=dry_run)
-        if match or dry_run:
+        if match != QKeySequence.SequenceMatch.NoMatch or dry_run:
             return match
 
-        if keyutils.is_special(Qt.Key(e.key()), e.modifiers()):
+        try:
+            info = keyutils.KeyInfo.from_event(e)
+        except keyutils.InvalidKeyError as ex:
+            # See https://github.com/qutebrowser/qutebrowser/issues/7047
+            log.keyboard.debug(f"Got invalid key: {ex}")
+            return QKeySequence.SequenceMatch.NoMatch
+        if info.is_special():
             # this is not a proper register key, let it pass and keep going
-            return QKeySequence.NoMatch
+            return QKeySequence.SequenceMatch.NoMatch
 
         key = e.text()
 
@@ -295,4 +300,4 @@ class RegisterKeyParser(CommandKeyParser):
 
         self.request_leave.emit(
             self._register_mode, "valid register key", True)
-        return QKeySequence.ExactMatch
+        return QKeySequence.SequenceMatch.ExactMatch
